import sys
import os
import shutil


def exit_cli(print_func, message):
    print_func("%s" % message)
    sys.exit(0)


def display_file(file_name):
    """
    Open given file with default user program.
    """
    if sys.platform.startswith("linux"):
        os.system("xdg-open %s" % file_name)

    elif sys.platform.startswith("darwin"):
        os.system("open %s" % file_name)


def remove_dir(directory):
    for filename in os.listdir(directory):
        file_path = os.path.join(directory, filename)
        try:
            if os.path.isfile(file_path) or os.path.islink(file_path):
                os.unlink(file_path)
            elif os.path.isdir(file_path):
                shutil.rmtree(file_path)
        except Exception as e:
            print("Failed to delete %s. Reason: %s" % (file_path, e))
            exit_cli(print, "")



import click
import sys
from rasterio.errors import RasterioIOError
import pkg_resources

from felicette.utils.geo_utils import geocoder_util
from felicette.utils.file_manager import check_sat_path, file_paths_wrt_id
from felicette.sat_downloader import (
    download_landsat_data,
    search_satellite_data,
    preview_satellite_image,
    download_data,
)
from felicette.utils.sys_utils import exit_cli, remove_dir
from felicette.sat_processor import process_data


def trigger_download_and_processing(item, bands):
    # download data
    data_id = download_data(item, bands)
    # process data
    process_data(data_id, bands)


@click.command()
@click.option(
    "-c",
    "--coordinates",
    nargs=2,
    type=float,
    help="Coordinates in (lon, lat) format. This overrides -l command",
)
@click.option("-l", "--location-name", type=str, help="Location name in string format")
@click.option(
    "-pan",
    "--pan-enhancement",
    default=False,
    is_flag=True,
    help="Enhance image with panchromatic band",
)
@click.option(
    "--no-preview",
    default=False,
    is_flag=True,
    help="Skip previewing of pre-processed low resolution RGB satellite image.",
)
@click.option(
    "-v",
    "--vegetation",
    default=False,
    is_flag=True,
    help="Show Color Infrared image to highlight vegetation",
)
@click.option(
    "-V",
    "--version",
    default=False,
    is_flag=True,
    help="Show the version number and quit",
)
@click.option("-p", "--product", type=str, default="landsat", help="Product name 'landsat'/'sentinel'")
def main(coordinates, location_name, pan_enhancement, no_preview, vegetation, version, product):
    """Satellite imagery for dummies."""
    if version:
        version_no = pkg_resources.require("felicette")[0].version
        exit_cli(print, f"felicette {version_no}.")
    if not coordinates and not location_name:
        exit_cli(print, "Please specify either --coordinates or --location-name")
    if location_name:
        coordinates = geocoder_util(location_name)

    # unless specified, cloud_cover_lt is 10
    item = search_satellite_data(coordinates, 10, product=product)

    # check if directory exists to save the data for this product id
    check_sat_path(item._data["id"])

    # if preview option is set, download and preview image
    if not no_preview:
        preview_satellite_image(item)

    # set bands to process
    bands = [2, 3, 4]
    if pan_enhancement and (product != "sentinel"):
        bands.append(8)

    if vegetation:
        bands = [3, 4, 5]

    # NB: can't enable pan-enhancement with vegetation
    # NB: can't enable pan-enhancement with sentinel

    try:
        trigger_download_and_processing(item, bands)
    except RasterioIOError:
        response = input(
            "Local data for this location is corrupted, felicette will remove existing data to proceed, are you sure? [Y/n]"
        )
        if response in ["y", "Y", ""]:
            # remove file dir
            file_paths = file_paths_wrt_id(item._data["id"])
            remove_dir(file_paths["base"])
            # retry downloading and processing image with a clean directory
            trigger_download_and_processing(item, bands)
        elif response in ["n", "N"]:
            exit_cli(print, "")


if __name__ == "__main__":
    main()



import tempfile
import os
import requests
from tqdm import tqdm
from rich import print as rprint
import boto3

from felicette.constants import band_tag_map
from felicette.utils.sys_utils import exit_cli

workdir = os.path.join(os.path.expanduser("~"), "felicette-data")


def check_sat_path(id):
    data_path = os.path.join(workdir, id)

    if not os.path.exists(data_path):
        os.makedirs(data_path, exist_ok=True)


def hook(t):
    def inner(bytes_amount):
        t.update(bytes_amount)

    return inner


def save_to_file(url, filename, id, info_message, meta=None):
    product_type = get_product_type_from_id(id)
    data_path = os.path.join(workdir, id)
    data_id = filename.split("/")[-1].split("-")[1].split(".")[0]
    rprint(info_message)
    file_path = os.path.join(data_path, filename)

    if product_type == "sentinel" and meta:
        aws_access_key_id = os.environ.get("AWS_ACCESS_KEY_ID", None)
        aws_secret_access_key=os.environ.get("AWS_SECRET_ACCESS_KEY", None)
        # if access key or secret isn't defined, print error message and exit
        if (not aws_access_key_id) or (not aws_secret_access_key):
            exit_cli(rprint, "Error: [red]AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY[/red] must be set in environment variables to access Sentinel data.")
        # prepare boto3 client
        s3_client = boto3.Session().client(
            "s3",
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key,
        )
        band = os.path.join(meta["path"], "B0%s.jp2" % (meta["band_id"]))
        filesize = s3_client.head_object(
            Bucket="sentinel-s2-l1c", Key=band, RequestPayer="requester"
        ).get("ContentLength")
        with tqdm(total=filesize, unit="B", unit_scale=True, desc=data_id) as t:
            response = s3_client.download_file(
                Bucket="sentinel-s2-l1c",
                Key=band,
                Filename=file_path,
                ExtraArgs={"RequestPayer": "requester"},
                Callback=hook(t),
            )
    else:
        # for landsat, and preview images - resources which can be downloaded via http
        response = requests.get(url, stream=True)
        with tqdm.wrapattr(
            open(file_path, "wb"),
            "write",
            miniters=1,
            desc=data_id,
            total=int(response.headers.get("content-length", 0)),
        ) as fout:
            for chunk in response.iter_content(chunk_size=4096):
                fout.write(chunk)
        fout.close()


def data_file_exists(filename):
    return os.path.exists(filename)


def get_product_type_from_id(id):
    if "LC" in id:
        return "landsat"
    else:
        return "sentinel"


def file_paths_wrt_id(id):
    home_path_id = os.path.join(workdir, id)
    extension = None
    if get_product_type_from_id(id) == "landsat":
        extension = "tiff"
    else:
        extension = "jp2"
    return {
        "base": home_path_id,
        "preview": os.path.join(home_path_id, "%s-preview.jpg" % (id)),
        "b5": os.path.join(home_path_id, "%s-b5.%s" % (id, extension)),
        "b4": os.path.join(home_path_id, "%s-b4.%s" % (id, extension)),
        "b3": os.path.join(home_path_id, "%s-b3.%s" % (id, extension)),
        "b2": os.path.join(home_path_id, "%s-b2.%s" % (id, extension)),
        "b8": os.path.join(home_path_id, "%s-b8.%s" % (id, extension)),
        "stack": os.path.join(home_path_id, "%s-stack.tiff" % (id)),
        "pan_sharpened": os.path.join(home_path_id, "%s-pan.tiff" % (id)),
        "output_path": os.path.join(
            home_path_id, "%s-color-processed.tiff" % (id)
        ),
        "output_path_jpeg": os.path.join(
            home_path_id, "%s-color-processed.jpeg" % (id)
        ),
        "vegetation_path": os.path.join(
            home_path_id, "%s-vegetation.tiff" % (id)
        ),
        "vegetation_path_jpeg": os.path.join(home_path_id, "%s-vegetation.jpeg" % (id)),
    }



"""
Satellite imagery for dummies.
"""
from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

dependencies = [
    "click==7.1.2",
    "requests==2.24.0",
    "sat-search==0.2.3",
    "rich==3.3.2",
    "tqdm==4.48.0",
    "rasterio==1.1.5",
    "rio-color==1.0.0",
    "pillow==7.2.0",
    "opencv-python==4.3.0.36",
    "boto3==1.14.21"
]

setup(
    name="felicette",
    version="0.1.12",
    url="https://github.com/plant99/felicette",
    license="MIT",
    author="Shivashis Padhi",
    author_email="shivashispadhi@gmail.com",
    description="Satellite imagery for dummies.",
    long_description=__doc__,
    packages=find_packages(exclude=["tests"]),
    include_package_data=True,
    zip_safe=False,
    platforms="any",
    install_requires=dependencies,
    entry_points={"console_scripts": ["felicette = felicette.cli:main",],},
    classifiers=[
        # As from http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 3 - Alpha",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: Unix",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
